/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

class KeycloakRealmMapperTest {

	@Spy
	private final KeycloakRealmMapper mapper = Mappers.getMapper(KeycloakRealmMapper.class);

	@Test
	void shouldMapDisplayName() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getDisplayName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.DISPLAY_NAME);
	}

	@Test
	void shouldBeEnabled() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.isEnabled()).isTrue();
	}

	@Test
	void shouldBeResetPasswordAllowed() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.isResetPasswordAllowed()).isTrue();
	}

	@Test
	void shouldBeSupportedLocaleDe() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getSupportedLocales()).containsExactly("de");
	}

	@Test
	void shouldBeDefaultLocaleDe() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getDefaultLocale()).isEqualTo("de");
	}

	@Test
	void shouldBeInternationalizationEnabled() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.isInternationalizationEnabled()).isTrue();
	}

	@Test
	void checkPasswordPolicy() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getPasswordPolicy()).isEqualTo("upperCase(1) and lowerCase(1) and length(8) and notUsername");
	}

	@Test
	void shouldSetActionTokenGeneratedByUserLifespan() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getActionTokenGeneratedByUserLifespan()).isEqualTo(900);
	}

	@Test
	void shouldMapSmtpServer() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getSmtpServer().size()).isEqualTo(8);
	}

	@Test
	void shouldContainSmtpServerKeysValues() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getSmtpServer())
				.containsEntry("user", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_USER)
				.containsEntry("password", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PASSWORD)
				.containsEntry("host", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_HOST)
				.containsEntry("port", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PORT)
				.containsEntry("starttls", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_STARTTLS)
				.containsEntry("auth", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_AUTH)
				.containsEntry("from", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM)
				.containsEntry("fromDisplayName", KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM_DISPLAY_NAME);
	}

	@Test
	void shouldMapWhenSmtpServerIsNull() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.createBuilder().smtpServer(null).build());

		assertThat(mapped.getSmtpServer()).isEmpty();
	}

	@Test
	void shouldMapRealmRoles() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

		assertThat(mapped.getRoles().getRealm()).hasSize(2);
	}

	@Test
	void shouldContainRealmRolesValues() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());
		var mappedRealmRoles = mapped.getRoles();

		assertThat(mappedRealmRoles.getRealm().get(0).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_1);
		assertThat(mappedRealmRoles.getRealm().get(1).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_2);
	}
}
